package org.spring.jmsinterop.builders;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.text.NumberFormat;

import javax.jms.MapMessage;
import javax.jms.Session;

import org.spring.jmsinterop.converters.ConversionContext;
import org.spring.jmsinterop.util.Fields;
import org.spring.jmsinterop.util.Messages;

public class BigDecimalBuilder implements Builder {

	private static int maximumFractionDigits = 8;
	private static final ThreadLocal threadLocal = new ThreadLocal();

	 
	public void addValue(MapMessage mm, Session session,
			ConversionContext conversionContext, String fieldName, Class fieldClass,
			Object fieldVal) {
		Messages.setMessageObject(mm, fieldName, 
								  getBigDecimalFormatter().format(fieldVal));

	}

	public boolean canBuild(Class targetClass) {
		return targetClass.equals(BigDecimal.class);
	}

	public void setValue(ConversionContext conversionContext, Object targetObject,
			Object val, Field targetField) {
		Fields.write(targetField, targetObject, new BigDecimal(val.toString()));
	}
	
    /**
     * Get the number formatter. This returns a Threadsafe <tt>NumberFormat</tt>
     * instance.
     * 
     * @return Threadsafe <tt>NumberFormat</tt> instance.
     *         <code>GroupingUsed</code> is set to <code>false</code> and
     *         <code>MaximumFractionDigits</code> is set to
     *         <code>maximumFractionDigits</code>.
     * @see #setMaximumFractionDigits(int)
     */
    protected static NumberFormat getBigDecimalFormatter()
    {
        NumberFormat numberFmt = (NumberFormat) threadLocal
                .get();
        if (numberFmt == null)
        {
            numberFmt = NumberFormat.getNumberInstance();
            numberFmt.setGroupingUsed(false);
            numberFmt.setMaximumFractionDigits(maximumFractionDigits);
            threadLocal.set(numberFmt);
        }
        return numberFmt;
    }
   
}
