package org.spring.jmsinterop.builders;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Session;

import org.spring.jmsinterop.converters.CannotResolveClassException;
import org.spring.jmsinterop.converters.ConversionContext;
import org.spring.jmsinterop.util.Fields;
import org.spring.jmsinterop.util.Messages;
import org.springframework.beans.BeanUtils;
import org.springframework.jms.support.JmsUtils;
import org.springframework.jms.support.converter.MessageConversionException;
import org.springframework.jms.support.converter.MessageConverter;
import org.springframework.util.ClassUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.ReflectionUtils;

public class CollectionBuilder implements Builder {

	// TODO move this responsibility elsewhere - an unmarshalling context.
	private DefaultImplementationMapping implMapping;

	public CollectionBuilder() {
		implMapping = new DefaultImplementationMapping();
	}

	public boolean canBuild(Class targetClass) {
		return Collection.class.isAssignableFrom(targetClass);
	}

	public void addValue(MapMessage mm, Session session,
			ConversionContext conversionContext, String fieldName, Class fieldClass,
			Object fieldVal) {
		if (fieldVal instanceof Collection) {
			Collection collection = (Collection) fieldVal;
			MapMessage collectionMsg = Messages.createMapMessage(session);
			Messages.setBooleanProperty(collectionMsg, "JMS_TIBCO_MSG_EXT",
					true);
			Messages.setMessageObject(mm, fieldName, collectionMsg);
			int i = 0;
			for (Iterator iter = collection.iterator(); iter.hasNext();) {
				Object objectToConvert = iter.next();
				try {
					Messages.setMessageObject(collectionMsg, Integer
							.toString(i), 
							conversionContext.convertToMessage(objectToConvert, session));
				} catch (JMSException e) {
					JmsUtils.convertJmsAccessException(e);
				}
				i++;
			}
		}

	}

	public void setValue(ConversionContext conversionContext, Object targetObject,
			Object val, Field targetField) {

		if (val instanceof MapMessage) {
			MapMessage mm = (MapMessage) val;
			Collection collection = (Collection) implMapping.createCollection(targetField);

			Fields.write(targetField, targetObject, populateCollection(
					collection, conversionContext, mm));
		}

	}



	private Object populateCollection(Collection collection,
			ConversionContext conversionContext, MapMessage mm) {

		// TODO consider encoding the length in the message.
		int numFields = 0;
		try {
			for (Enumeration e = mm.getMapNames(); e.hasMoreElements();) {
				numFields++;
				e.nextElement();
			}

			for (int i = 0; i < numFields; i++) {
				String jmsFieldName = Integer.toString(i);
				Object val = null;
				if (mm.itemExists(jmsFieldName)) {
					val = mm.getObject(jmsFieldName);
				}
				if (val != null)
				{
					if (val instanceof MapMessage)
					{
						MapMessage nestedMapMessage = (MapMessage)val;
						Object targetObject = conversionContext.convertToObject(nestedMapMessage);
						collection.add(targetObject);
					}
					else
					{
						 throw new MessageConversionException("Could not convert message for collection type target.");		                    
					}
					
				}

			}
		} catch (JMSException e) {
			JmsUtils.convertJmsAccessException(e);
		}
		return collection;

	}


}
