package org.spring.jmsinterop.builders;

import java.lang.reflect.Field;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Session;

import org.spring.jmsinterop.converters.ConversionContext;
import org.spring.jmsinterop.util.Fields;
import org.spring.jmsinterop.util.Messages;
import org.springframework.jms.support.JmsUtils;
import org.springframework.jms.support.converter.MessageConversionException;

public class MapBuilder implements Builder {

	// TODO move this responsibility elsewhere - an unmarshalling context.
	private DefaultImplementationMapping implMapping;

	public MapBuilder() {
		implMapping = new DefaultImplementationMapping();
	}

	public boolean canBuild(Class targetClass) {
		return Map.class.isAssignableFrom(targetClass);
	}

	public void addValue(MapMessage mm, Session session,
			ConversionContext conversionContext, String fieldName,
			Class fieldClass, Object fieldVal) {

		if (fieldVal instanceof Map) {
			Map map = (Map) fieldVal;

			MapMessage messageToPopulate = null;
			//Map nested inside a contained object being sent.
			if (fieldName != null) {

				MapMessage mapMsg = Messages.createMapMessage(session);
				Messages.setBooleanProperty(mapMsg, "JMS_TIBCO_MSG_EXT", true);
				Messages.setMessageObject(mm, fieldName, mapMsg);
				messageToPopulate = mapMsg;
			} else {
				messageToPopulate = mm;
			}

			Messages.setMessageObject(messageToPopulate, conversionContext
					.getClassMapper().getClassIdFieldName(), conversionContext
					.getClassMapper().fromClass(map.getClass()));

			for (Iterator it = map.entrySet().iterator(); it.hasNext();) {
				Map.Entry entry = (Map.Entry) it.next();
				if (!(entry.getKey() instanceof String)) {
					throw new MessageConversionException(
							"Cannot convert non-String key of type ["
									+ (entry.getKey() != null ? entry.getKey()
											.getClass().getName() : null)
									+ "] to MapMessage entry");
				}

				conversionContext.convertToMessage(messageToPopulate, session,
						entry.getKey().toString(), entry
								.getValue());

			}
		}

	}

	public void setValue(ConversionContext conversionContext,
			Object targetObject, Object val, Field targetField) {
		if (val instanceof MapMessage) {
			MapMessage mm = (MapMessage) val;
			Map map = (Map) implMapping.createMap(targetField);

			Fields.write(targetField, targetObject, populateMap(map,
					conversionContext, mm));
		}

	}

	private Object populateMap(Map map, ConversionContext conversionContext,
			MapMessage mm) {
		try {
			Enumeration en = mm.getMapNames();
			while (en.hasMoreElements()) {
				String key = (String) en.nextElement();
				if (!key.equals("__TypeId__")) {
					map.put(key, mm.getObject(key));
				}
			}

		} catch (JMSException e) {
			JmsUtils.convertJmsAccessException(e);
		}
		return map;
	}

}
