package org.spring.jmsinterop.builders;


import java.lang.reflect.Field;

import javax.jms.MapMessage;
import javax.jms.Session;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import org.spring.jmsinterop.converters.ConversionContext;
import org.spring.jmsinterop.util.Fields;
import org.spring.jmsinterop.util.Messages;
import org.springframework.jms.support.converter.MessageConversionException;

public class TimestampBuilder implements Builder {

    private static final String PATTERN = "dd MMM yyyy HH:mm:ss,SSS z";

    private static ThreadLocal threadLocal = new ThreadLocal();


	public boolean canBuild(Class targetClass) {
		return targetClass.equals(Timestamp.class);
	}
	
	public void addValue(MapMessage mm, Session session,
			ConversionContext conversionContext, String fieldName, Class fieldClass,
			Object fieldVal) {
        Timestamp realVal = (Timestamp) fieldVal;
        String sVal = getDateFormat().format(new Date(realVal.getTime()));
		Messages.setMessageObject(mm, fieldName, (String) sVal);

	}



	public void setValue(ConversionContext conversionContext, Object targetObject,
			Object val, Field targetField) {
		
        try {
			Timestamp timestamp = new Timestamp(getDateFormat().parse(val.toString()).getTime());
			Fields.write(targetField, targetObject, timestamp);
		} catch (ParseException e) {
			throw new MessageConversionException("Could not convert Timestamp", e);
		}


	}
	
    private static SimpleDateFormat getDateFormat()
    {
        SimpleDateFormat dateFmt = (SimpleDateFormat) threadLocal.get();
        if (dateFmt == null)
        {
            dateFmt = new SimpleDateFormat(PATTERN);
            threadLocal.set(dateFmt);
        }
        dateFmt.setLenient(true);
        dateFmt.setTimeZone(TimeZone.getTimeZone("GMT"));
        return dateFmt;
    }

}
