package org.spring.jmsinterop.converters;

import java.lang.reflect.Field;
import java.util.IdentityHashMap;
import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.Session;

import org.spring.jmsinterop.builders.Builder;
import org.spring.jmsinterop.builders.BuilderLookup;
import org.spring.jmsinterop.util.Fields;
import org.springframework.beans.BeanUtils;
import org.springframework.jms.support.converter.MessageConversionException;

public class ConversionContext {
	
    private ClassMapper classMapper;
    
    private BuilderLookup builderLookup;
    
    private Map nodeMap = new IdentityHashMap(113);
    
    
	public ConversionContext() {
		builderLookup = new BuilderLookup();
	}
    
	// Properties 


	public ClassMapper getClassMapper() {
		return classMapper;
	}

	public void setClassMapper(ClassMapper classMapper) {
		this.classMapper = classMapper;
	}

	public BuilderLookup getBuilderLookup() {
		return builderLookup;
	}


	public void setBuilderLookup(BuilderLookup builderLookup) {
		this.builderLookup = builderLookup;
	}
	
	
	// Methods
	
    public Message convertToMessage(Object objectToConvert, Session session) throws JMSException
    {
        MapMessage mm = session.createMapMessage();
        mm.setBooleanProperty("JMS_TIBCO_MSG_EXT", true);
        convertToMessage(mm, session, null, objectToConvert);
        return mm;
    }
    
    public void convertToMessage(MapMessage mm, Session session, 
            String name,
            Object objectToConvert)
    {
    	if (nodeMap.containsKey(objectToConvert))
    	{
    		throw new MessageConversionException("Circular reference for object = " + objectToConvert);
    	}
    	nodeMap.put(objectToConvert, "");
    	Builder builder = getBuilderLookup().getBuilderForClass(objectToConvert.getClass());
    	builder.addValue(mm, session, this, name, null, objectToConvert); 
    	nodeMap.remove(objectToConvert);
    }
    
    public Object convertToObject(MapMessage mm) throws JMSException
    {
    	Class targetClass = getTargetClass(mm);
    	Object targetObject = BeanUtils.instantiateClass(targetClass);
		//TODO refactor to class that manages/caches fields.
		Field[] fields = targetClass.getDeclaredFields();
		for (int i = 0; i < fields.length; i++) {
			
			Field targetField = fields[i];
	        if (!Fields.fieldModifiersSupported(targetField)) {
	            continue;
	        }
	        targetField.setAccessible(true);
	        
            this.convertToObject(mm, fields[i], targetObject);
		}
		return targetObject;
    }

	private void convertToObject(MapMessage mm, Field targetField, Object targetObject) throws JMSException {

        String fieldName = targetField.getName();
        Object val = null;

        if (mm.itemExists(fieldName))
        {
            val = mm.getObject(fieldName);
        }
        if (val != null)
        {
            Class valClass = targetField.getType();
            Builder builder = builderLookup.getBuilderForClass(valClass);
            builder.setValue(this, targetObject, val, targetField);
        }
		
	}

	private Class getTargetClass(MapMessage mm) throws JMSException {
		if (mm.itemExists(classMapper.getClassIdFieldName())) {
			return classMapper.toClass(mm.getString(classMapper
					.getClassIdFieldName()));
		} else {
			throw new MessageConversionException(
					"Could not find type ID field with name "
							+ classMapper.getClassIdFieldName()
							+ " in message.");
		}
	}
}
