package org.spring.jmsinterop.converters;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.jms.MapMessage;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.util.ClassUtils;

public class SimpleClassMapper implements ClassMapper, InitializingBean {

    private String defaultPackage;

    //Generics not used to easily support both 1.1 and 2.0
    private Map idClassMapping;
    private Map classIdMapping;
    
    private String defaultHashtableTypeId = "Hashtable"; 
    
    private Class defaultHashtableClass = HashMap.class;
    
    
	public SimpleClassMapper() {
		idClassMapping = new HashMap();
		classIdMapping = new HashMap();
		
	}


	public Map getIdClassMapping() {
		return idClassMapping;
	}



	public void setIdClassMapping(Map idClassMapping) {
		this.idClassMapping = idClassMapping;
	}


	public String getDefaultPackage() {
		return defaultPackage;
	}


	public void setDefaultPackage(String defaultPackage) {
		this.defaultPackage = defaultPackage;
	}


	public String getClassIdFieldName()
	{
		return "__TypeId__";
	}
	
	
	public void setDefaultHashtableClass(Class defaultHashtableClass) {
		this.defaultHashtableClass = defaultHashtableClass;
	}


	public String fromClass(Class classOfObjectToConvert) {
		if (classIdMapping.containsKey(classOfObjectToConvert))
		{
			return (String) classIdMapping.get(classOfObjectToConvert);
		}
		else
		{
			if (Map.class.isAssignableFrom(classOfObjectToConvert))
			{
				return defaultHashtableTypeId;
			}
			return ClassUtils.getShortName(classOfObjectToConvert);
		}
	}



	public Class toClass(String classId) {
		if (idClassMapping.containsKey(classId))
		{
			return (Class)idClassMapping.get(classId);
		}
		else
		{
			if (classId.equals(defaultHashtableTypeId))
			{
				return defaultHashtableClass;
			}
			String fqnName = defaultPackage + "." + classId;
			try {
				return ClassUtils.forName(fqnName);
			} catch (ClassNotFoundException e) {
				throw new CannotResolveClassException("Can not resolve " + fqnName, e);
			}
		}
	}

	public void afterPropertiesSet() throws Exception {
		validateClassMapping();
		
		
	}


	private void validateClassMapping() {
		Map finalIdClassMapping = new HashMap();
		for (Iterator iterator = idClassMapping.entrySet().iterator(); iterator.hasNext();) {
	      Map.Entry entry = (Map.Entry) iterator.next();
	      String id = entry.getKey().toString();
	      Class clazz = null;
	      if (entry.getValue() instanceof Class)
	      {
	    	  clazz = (Class)entry.getValue();  
	      }
	      else
	      {
	    	  try {
				clazz = ClassUtils.forName(entry.getValue().toString());
			} catch (ClassNotFoundException e) {
				throw new CannotResolveClassException("Can not resolve " + entry.getValue().toString(), e);				
			}
	      }
	      finalIdClassMapping.put(id, clazz);
    	  classIdMapping.put(clazz, id);	  
		}
		idClassMapping = finalIdClassMapping;
		
	}



}
