package org.spring.jmsinterop.converters;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.Session;

import org.spring.jmsinterop.bo.TradeRequest;
import org.springframework.jms.support.converter.MessageConversionException;
import org.springframework.jms.support.converter.MessageConverter;
import org.springframework.util.Assert;

public class TradeRequestConverter implements MessageConverter {

	public Object fromMessage(Message message) throws JMSException,
			MessageConversionException {
		Assert.isInstanceOf(MapMessage.class, message);
		
		MapMessage mm = (MapMessage)message;
		TradeRequest tradeRequest = new TradeRequest();
		try
		{
			tradeRequest.setAccountName(mm.getString("accountName"));
			tradeRequest.setBuyRequest(mm.getBoolean("buyRequest"));
			tradeRequest.setOrderType(mm.getString("orderType"));
			tradeRequest.setPrice(mm.getDouble("price"));
			tradeRequest.setQuantity(mm.getLong("quantity"));
			tradeRequest.setRequestId(mm.getString("requestId"));
			tradeRequest.setTicker(mm.getString("ticker"));
			tradeRequest.setUserName(mm.getString("userName"));
			return tradeRequest;
		} catch (JMSException e)
		{
            throw new MessageConversionException(
                    "Could not convert message to TradeRequest object",
                    e);	
		}	
	}

	public Message toMessage(Object objectToConvert, Session session) throws JMSException,
			MessageConversionException {
		
		//Most likely not used but implemented for completeness.
		Assert.isInstanceOf(TradeRequest.class, objectToConvert);
		
		try
		{
			MapMessage mm = session.createMapMessage();
			TradeRequest tradeRequest = (TradeRequest) objectToConvert;
			
			mm.setString("accountName", tradeRequest.getAccountName());
			mm.setBoolean("buyRequest", tradeRequest.isBuyRequest());
			mm.setString("orderType", tradeRequest.getOrderType());
			mm.setDouble("price", tradeRequest.getPrice());
			mm.setLong("quantity", tradeRequest.getQuantity());
			mm.setString("requestId", tradeRequest.getRequestId());
			mm.setString("ticker", tradeRequest.getTicker());
			mm.setString("username", tradeRequest.getUserName());
			
			return mm;
		} catch (JMSException e)
		{
			throw new MessageConversionException("Could not convert TradeRequest object to message", e);
		}

	}

}
