package org.spring.jmsinterop.handlers;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.spring.jmsinterop.bo.*;
import org.spring.jmsinterop.gateways.UserGateway;

public class StockAppHandler {

	protected final Log logger = LogFactory.getLog(getClass());

	private UserGateway userGateway;

	public UserGateway getUserGateway() {
		return userGateway;
	}

	public void setUserGateway(UserGateway userGateway) {
		this.userGateway = userGateway;
	}

	public void handleObject(TradeRequest tradeRequest) {
		logger.info("Recieved TradeRequest.");
		sleep();
		Trade trade = createTrade(tradeRequest);
		userGateway.send(trade);
	}

	public void handleObject(PortfolioRequest portfolioRequest) {
		logger.info("Recieved PortfolioRequest.");
		sleep();
		Portfolio portfolio = createPortfolio(portfolioRequest);		
		userGateway.send(portfolio);
	}
	
	private void sleep() {
		logger.info("sleeping for 2 seconds to simulate server processing...");
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private Portfolio createPortfolio(PortfolioRequest portfolioRequest) {
		Portfolio portfolio = new Portfolio();
		portfolio.setUser(portfolioRequest.getUser());
		portfolio.setPortfolioName("Joe's Trading Account");

		Trade trade = createTrade();
		Trade anotherTrade = createAnotherTrade();
		portfolio.getTrades().add(trade);
		portfolio.getTrades().add(anotherTrade);
		return portfolio;
	}

	private Trade createTrade(TradeRequest tradeRequest) {
		Trade trade = new Trade();
		trade.setOrderType(tradeRequest.getOrderType());
		if (tradeRequest.getOrderType().compareTo("MARKET") == 0) {
			trade.setPrice(27.6);
		} else {
			trade.setPrice(tradeRequest.getPrice());
		}
		trade.setQuantity(tradeRequest.getQuantity());
		trade.setTicker(tradeRequest.getTicker());

		return trade;
	}

	private Trade createTrade() {
		Trade trade = new Trade();
		trade.setOrderType("Market");
		trade.setPrice(28.3);
		trade.setQuantity(1000);
		trade.setTicker("CSCO");
		return trade;
	}
	private Trade createAnotherTrade() {
		Trade trade = new Trade();
		trade.setOrderType("Market");
		trade.setPrice(30.14);
		trade.setQuantity(2500);
		trade.setTicker("MSFT");
		return trade;
	}
}
