

using System;
using System.Globalization;
using System.Reflection;
using Spring.JmsInterop.Converters;
using TIBCO.EMS;

namespace Spring.JmsInterop.Builders
{
    public class DateTimeBuilder : IBuilder
    {
        private DateTimeFormatInfo dateTimeFormatInfo;

        protected string fullDateTimePattern = "dd MMM yyyy HH':'mm':'ss','fff 'GMT'";


        public DateTimeBuilder()
        {
            dateTimeFormatInfo = new DateTimeFormatInfo();
            dateTimeFormatInfo.FullDateTimePattern = fullDateTimePattern;
        }
        
        

        public bool CanBuild(Type targetType)
        {
            return targetType.Equals(typeof(DateTime)); 
        }

        public void AddValue(MapMessage mm, Session session, ConversionContext conversionContext, string fieldName, Type fieldType, object fieldVal)
        {
            DateTime realVal = (DateTime)fieldVal;
            if (realVal.CompareTo(DateTime.MinValue) != 0)
            {
                mm.SetString(fieldName, DateToString(realVal));
            }
        }

        

        public void SetValue(MapMessage mapMessage, ConversionContext conversionContext, object targetObject, object fieldVal, FieldInfo targetFieldInfo)
        {
            DateTime dt = StringToDate(fieldVal.ToString());
            targetFieldInfo.SetValue(targetObject, dt);
        }

        protected virtual DateTime StringToDate(string dateString)
        {
            return DateTime.ParseExact(dateString, "F", dateTimeFormatInfo, DateTimeStyles.NoCurrentDateDefault | DateTimeStyles.AllowWhiteSpaces).ToLocalTime();            
        }


        protected virtual string DateToString(DateTime dateTime)
        {
            return dateTime.ToUniversalTime().ToString(dateTimeFormatInfo.FullDateTimePattern);
        }
    }
}
