using System;
using System.Reflection;
using Spring.JmsInterop.Converters;
using Spring.JmsInterop.Util;
using TIBCO.EMS;

namespace Spring.JmsInterop.Builders
{
    public class ObjectBuilder : IBuilder
    {
        public bool CanBuild(Type targetType)
        {
            return true;
        }

        public void AddValue(MapMessage mm, 
                             Session session,
                             ConversionContext conversionContext, 
                             string name, 
                             Type type, //null
                             object objectToConvert)
        {
            MapMessage messageToPopulate = null;
            //TODO use conversion context to indicate start of processing instead of check for null
            if (name != null)
            {
                MapMessage nestedMapMessage = session.CreateMapMessage();
                nestedMapMessage.SetBooleanProperty("JMS_TIBCO_MSG_EXT", true);
                mm.SetObject(name, nestedMapMessage);
                messageToPopulate = nestedMapMessage;
            }
            else
            {
                messageToPopulate = mm;
            }
            
                       
            //mm.SetObject(fieldName, conversionContext.ToMessage(fieldVal, session));

            messageToPopulate.SetString(conversionContext.TypeMapper.TypeIdFieldName, 
                                        conversionContext.TypeMapper.FromType(objectToConvert.GetType()));
          
            FieldInfo[] fields = ReflectionUtils.GetFields(objectToConvert.GetType());
            foreach (FieldInfo fi in fields)
            {
                //fieldname, fieldtype, owningobject.
                string fieldName = fi.Name;
                object fieldVal =  fi.GetValue(objectToConvert);
                if (fieldVal != null)
                {
                    conversionContext.ConvertToMessage(messageToPopulate, session,
                                      fieldName, fieldVal);   
                }

            }
            
        }

        public void SetValue(MapMessage mapMessage, ConversionContext conversionContext, 
                             object targetObject, object val, FieldInfo targetFieldInfo)
        {
            MapMessage mm = val as MapMessage;
            if (mm != null)
            {
                targetFieldInfo.SetValue(targetObject, conversionContext.ConvertToObject(mm));
            }
        }

    }
}
