using System;
using System.Reflection;
using Spring.JmsInterop.Builders;
using Spring.JmsInterop.Util;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using Spring.Objects;
using TIBCO.EMS;

namespace Spring.JmsInterop.Converters
{
    public class ConversionContext
    {
        
        private ITypeMapper typeMapper;

        private BuilderLookup builderLookup;

        private ObjectIdentityLookup objectIdentityLookup;


        public ConversionContext()
        {
            builderLookup = new BuilderLookup();
            objectIdentityLookup = new ObjectIdentityLookup();
        }


        #region Properties
        
        public ITypeMapper TypeMapper
        {
            get { return typeMapper; }
            set { typeMapper = value; }
        }


        public BuilderLookup BuilderLookup
        {
            get { return builderLookup; }
            set { builderLookup = value; }
        }

        #endregion 

        #region Methods
        
        public Message ConvertToMessage(object objectToConvert, Session session)
        {

            MapMessage mm = session.CreateMapMessage();
            mm.SetBooleanProperty("JMS_TIBCO_MSG_EXT", true);
            ConvertToMessage(mm, session, null, objectToConvert);
            return mm;
        }
        
        public void ConvertToMessage(MapMessage mm, Session session, string name, object objectToConvert)
        {
            if (objectIdentityLookup.Contains(objectToConvert))
            {
                throw new MessageConversionException("Circular reference for object = " + objectToConvert);
            }
            objectIdentityLookup.Add(objectToConvert);
            IBuilder builder = BuilderLookup.GetBuilderForType(objectToConvert.GetType());
            builder.AddValue(mm, session, this, name, null, objectToConvert);             
            objectIdentityLookup.Remove(objectToConvert);
        }

        
        public object ConvertToObject(MapMessage mm)
        {
            Type targetType = GetTargetType(mm);
            object targetObject = ObjectUtils.InstantiateType(targetType);
            FieldInfo[] fields = ReflectionUtils.GetFields(targetType);
            foreach (FieldInfo targetFieldInfo in fields)
            {

                this.ConvertToObject(mm, targetFieldInfo, targetObject);

            }
            return targetObject;
        }



        private void ConvertToObject(MapMessage mm, FieldInfo targetFieldInfo, object targetObject)
        {
            string fieldName = targetFieldInfo.Name;
            object val = null;

            if (mm.ItemExists(fieldName))
            {
                val = mm.GetObject(fieldName);
            }
            if (val != null)
            {
                Type valType = targetFieldInfo.FieldType;
                IBuilder builder = BuilderLookup.GetBuilderForType(valType);
                builder.SetValue(null, this, targetObject, val, targetFieldInfo);
            }
        }


        private Type GetTargetType(MapMessage mm)
        {
            if (mm.ItemExists(TypeMapper.TypeIdFieldName))
            {
                return TypeMapper.ToType(mm.GetString(TypeMapper.TypeIdFieldName));
            }
            else
            {
                throw new MessageConversionException("Could not find type ID field with name " +
                                                     TypeMapper.TypeIdFieldName + " in message.");
            }
        }

        #endregion
        
    }
}
