using System;
using System.Collections;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Converters
{
    /// <summary>
    /// Converts messages using either the SimpleMessageConverter or
    /// ReflectionMessageConverter.
    /// </summary>
    /// <remarks>
    /// More advanced implementations could be designed using the chain of responsibility
    /// design pattern.
    /// </remarks>
    public class SimpleMultiMessageConverter : IMessageConverter
    {
        private IMessageConverter defaultMessageConverter = new SimpleMessageConverter();
        private ReflectionMessageConverter reflectionMessageConverter;
        
        public ReflectionMessageConverter ReflectionMessageConverter
        {
            set { reflectionMessageConverter = value; }
        }

        public Message ToMessage(object objectToConvert, Session session)
        {
            if (objectToConvert.GetType().Equals(typeof(string)) ||
                typeof(IDictionary).IsAssignableFrom(objectToConvert.GetType()) ||
                objectToConvert.GetType().Equals(typeof(Byte[])))                
            {
                return defaultMessageConverter.ToMessage(objectToConvert, session);
            }
            else
            {
                return reflectionMessageConverter.ToMessage(objectToConvert, session);
            }
        }

        public object FromMessage(Message message)
        {
            if (message.GetStringProperty(reflectionMessageConverter.ConverterIdFieldName) != null)
            {
                return reflectionMessageConverter.FromMessage(message);
            }
            else
            {
                return defaultMessageConverter.FromMessage(message);
            }
                
        }
    }
}
