
using System;
using Spring.JmsInterop.Bo;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Converters
{
    public class TradeRequestConverter : IMessageConverter 
    {
        public Message ToMessage(object objectToConvert, Session session)
        {
            TradeRequest tradeRequest = objectToConvert as TradeRequest;
            if (tradeRequest == null)
            {
                throw new MessageConversionException("TradeRequestConverter can not convert object of type " +
                                                     objectToConvert.GetType());
            }

            try
            {
                MapMessage mm = session.CreateMapMessage();

                mm.SetString("accountName", tradeRequest.AccountName);
                mm.SetBoolean("buyRequest", tradeRequest.BuyRequest);
                mm.SetString("orderType", tradeRequest.OrderType);
                mm.SetDouble("price", tradeRequest.Price);
                mm.SetLong("quantity", tradeRequest.Quantity);
                mm.SetString("requestId", tradeRequest.RequestId);
                mm.SetString("ticker", tradeRequest.Ticker);
                mm.SetString("username", tradeRequest.UserName);

                return mm;
                
            } catch (Exception e)
            {
                throw new MessageConversionException("Could not convert TradeRequest to message", e);
            }
        }

        public object FromMessage(Message message)
        {
            //Most likely not used by the client but implemented for the sake of
            //completeness. 
            MapMessage mm = message as MapMessage;
            if (mm != null)
            {
                TradeRequest tradeRequest = new TradeRequest();
                tradeRequest.AccountName = mm.GetString("accountName");
                tradeRequest.BuyRequest = mm.GetBoolean("buyRequest");
                tradeRequest.OrderType = mm.GetString("orderType");
                tradeRequest.Price = mm.GetDouble("price");
                tradeRequest.Quantity = mm.GetLong("quantity");
                tradeRequest.RequestId = mm.GetString("requestId");
                tradeRequest.Ticker = mm.GetString("ticker");
                tradeRequest.UserName = mm.GetString("username");

                return tradeRequest;
            }
            else
            {
                throw new MessageConversionException("Not of expected type MapMessage.  Message = " + message);
            }
        }
    }
}
