using System;
using System.Collections;
using NUnit.Framework;
using Spring.JmsInterop.Bo;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Converters
{
    [TestFixture]
    public class NewReflectionConverterTests
    {
        protected Session session;
        protected Connection connection;

        [SetUp]
        public void SetUp()
        {
            ConnectionFactory factory = new ConnectionFactory("tcp://localhost:7222");

            connection = factory.CreateConnection();

            session = connection.CreateSession(false, Session.AUTO_ACKNOWLEDGE);
        }
        
        [Test]
        public void ExpectExceptionDueToCircularReference()
        {
            Person p1 = new Person();
            Person p2 = new Person();
            p1.FirstName = "Joe";
            p2.FirstName = "Mary";
            p1.Friend = p2;
            p2.Friend = p1;

            IMessageConverter converter = GetReflectionMessageConverterTestAssembly();
            
            //didn't use attribute since don't know full message text.
            try
            {
                Message message = converter.ToMessage(p1, session);
                Assert.Fail("Expected exception due to Circular reference.");
            } catch (MessageConversionException e)
            {
                Assert.IsTrue(e.InnerException.Message.Contains("Circular"));
            }
        }
        
        [Test]
        public void PersonTest()
        {
            Person person = new Person();
            person.FirstName = "Joe";
            person.Age = 31;
            
            Person friend = new Person();
            friend.FirstName = "Mary";
            friend.Age = 32;
            person.Friend = friend;

            IDictionary favThings = new Hashtable();
            favThings.Add("my", "stuff");
            person.FavoriteThings = favThings;
            
            IMessageConverter converter = GetReflectionMessageConverterTestAssembly();

            Message message = converter.ToMessage(person, session);
            MapMessage mm = message as MapMessage;
            Assert.IsNotNull(mm);
            
            Assert.AreEqual("Joe", mm.GetString("firstName"));
            Assert.AreEqual(31, mm.GetInt("age"));
            
            
            /*             
            Assert.AreEqual("24 Dec 1969 11:34:21,233 GMT", mm.GetString("birthDate"));
            Assert.AreEqual(favNumbers, mm.GetObject("favNumbers") as int[]);
            */

            object personObj = converter.FromMessage(mm);
            Assert.IsInstanceOfType(typeof(Person), personObj);
            person = personObj as Person;
            Assert.AreEqual("Joe", person.FirstName);
            Assert.AreEqual(31, person.Age);

        }

        [Test]
        public void TestPerson()
        {
            Person person = new Person();
            person.AccountName = "ACCT-123";
            person.FirstName = "Joe";
            DateTime dt = new DateTime(1969, 12, 24, 6, 34, 21, 233);
            person.BirthDate = dt;
            int[] favNumbers = new int[] { 1, 2, 3 };
            person.FavNumbers = favNumbers;


            Hashtable things = new Hashtable();
            things.Add("my", "stuff");
            person.FavoriteThings = things;

            IMessageConverter converter = GetReflectionMessageConverterTestAssembly();

            Message message = converter.ToMessage(person, session);
            MapMessage mm = message as MapMessage;
            Assert.IsNotNull(mm);
            Assert.AreEqual("ACCT-123", mm.GetString("accountName"));

            Assert.AreEqual("24 Dec 1969 11:34:21,233 GMT", mm.GetString("birthDate"));
            Assert.AreEqual(favNumbers, mm.GetObject("favNumbers") as int[]);

            person = converter.FromMessage(mm) as Person;
            Assert.IsNotNull(person);
            Assert.AreEqual("ACCT-123", person.AccountName);
            Assert.AreEqual(dt, person.BirthDate);
            Assert.AreEqual(favNumbers, person.FavNumbers);

        }
        protected ReflectionMessageConverter GetReflectionMessageConverterTestAssembly()
        {
            ReflectionMessageConverter converter = new ReflectionMessageConverter();
            SimpleTypeMapper typeMapper = new SimpleTypeMapper();
            typeMapper.DefaultNamespace = "Spring.JmsInterop.Bo";
            typeMapper.DefaultAssemblyName = "JmsInterop.Integration.Tests";
            ConversionContext conversionContext = new ConversionContext();
            conversionContext.TypeMapper = typeMapper;
            converter.ConversionContext = conversionContext;

            return converter;
        }
    }
}
