

using System;
using System.Collections;
using NUnit.Framework;

namespace Spring.JmsInterop.Converters
{
    [TestFixture]
    public class SimpleTypeMapperTests
    {
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void NoDefaultAssemblyName()
        {
            SimpleTypeMapper mapper = new SimpleTypeMapper();
            mapper.DefaultNamespace = "Foo";
            mapper.AfterPropertiesSet();            
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void NoDefaultNamespace()
        {
            SimpleTypeMapper mapper = new SimpleTypeMapper();
            mapper.DefaultAssemblyName = "Foo";
            mapper.AfterPropertiesSet();
        }
        
        [Test]
        public void ValidSimpleConfiguration()
        {
            SimpleTypeMapper mapper = new SimpleTypeMapper();
            mapper.DefaultAssemblyName = "Spring.JmsInterop.Tests";
            mapper.DefaultNamespace = "Spring.JmsInterop.Converters";
            mapper.AfterPropertiesSet();
            PersonAssertions(mapper);
        }

        [Test]
        public void ValidMappingUsingType()
        {
            SimpleTypeMapper mapper = new SimpleTypeMapper();
            AddOrderMapping(mapper);
            mapper.AfterPropertiesSet();
            OrderAssertions(mapper);
        }

        [Test]
        public void ValidMappingUsingTypeAsString()
        {
            SimpleTypeMapper mapper = new SimpleTypeMapper();
            AddOrderMappingAsString(mapper);
            mapper.AfterPropertiesSet();
            OrderAssertions(mapper);
        }
        

        private static void OrderAssertions(SimpleTypeMapper mapper)
        {
            Assert.AreEqual("1",mapper.FromType(typeof(Order)) );
            Assert.IsTrue(mapper.ToType("1").Equals(typeof(Order)));
        }

        public void ValidCombination()
        {
            SimpleTypeMapper mapper = new SimpleTypeMapper();
            mapper.DefaultAssemblyName = "Spring.JmsInterop.Tests";
            mapper.DefaultNamespace = "Spring.JmsInterop.Converters";
            AddOrderMapping(mapper);
            mapper.AfterPropertiesSet();
            OrderAssertions(mapper);
            PersonAssertions(mapper);
        }

        private static void PersonAssertions(SimpleTypeMapper mapper)
        {
            Assert.AreEqual("Person", mapper.FromType(typeof(Person)));
            Assert.IsTrue(mapper.ToType("Person").Equals(typeof(Person)));
        }

        private static void AddOrderMapping(SimpleTypeMapper mapper)
        {
            IDictionary typeMapping = new Hashtable();
            typeMapping.Add("1", typeof(Order));
            mapper.IdTypeMapping = typeMapping;
        }

        private static void AddOrderMappingAsString(SimpleTypeMapper mapper)
        {
            IDictionary typeMapping = new Hashtable();
            typeMapping.Add("1", "Spring.JmsInterop.Converters.Order, Spring.JmsInterop.Tests");
            mapper.IdTypeMapping = typeMapping;
        }
    }
}
